#include <QCoreApplication>
#include <QApplication>
#include <QDebug>
#include <QTimer>
#include <QFileInfo>

#include "message_handler.h"
#include "interfaces.h"

#include "DCPlugin/include/hwapi.h"

#include <unistd.h>
#include <thread>
#include <memory>
#include <QSharedMemory>

#ifdef PTU5
#define SERIAL_PORT "ttymxc2"
#else
#define SERIAL_PORT "ttyUSB0"
#endif

static void updateBinary(std::unique_ptr<hwinf> hw,     // update d2dc*.bin
                         char const *fileToSendToDC,
                         char const *baudrate,
                         char const *serialInterface) {
    for (int i=0; i < 1;++i) {
        hw->dc_updateDC(fileToSendToDC, baudrate, serialInterface);
        std::this_thread::sleep_for(std::chrono::milliseconds(3000));
    }
    QCoreApplication::quit();
}

static void updatePrinterConf(std::unique_ptr<hwinf> hw, // update printer-file
                              QVector<int> nrOfTemplate,
                              QVector<QString> fileToSendToDC,
                              QString br, QString serial) {
    for (int i=0; i < 1;++i) {
        hw->dc_updatePrinterTemplate(hwapi::FileTypeJson::PRINTER,
                                     nrOfTemplate, fileToSendToDC,
                                     br, serial);

        hw->dc_printTemplate(hwapi::FileTypeJson::PRINTER,
                             nrOfTemplate, br, serial);
    }
    QCoreApplication::quit();
}

// argv[1]: file to send to dc
int main(int argc, char *argv[]) {
    QApplication a(argc, argv);

    if (!messageHandlerInstalled()) { // change internal qt-QDebug-handling
        atbInstallMessageHandler(atbDebugOutput);
        setDebugLevel(QtMsgType::QtDebugMsg);
        //setDebugLevel(QtMsgType::QtDebugMsg);
    }

    if (argc > 2) {
        qCritical() << "Usage: " << argv[0] << "<file to send to dc>";
        return -1;
    }

    std::unique_ptr<hwinf> hw(new hwapi());
    QFileInfo fileInfo(argv[1]);
    QString fname(fileInfo.fileName());
    int ret = 0;

    if (fname.startsWith("dc") && fname.endsWith(".bin")) {
        std::thread t(updateBinary, std::move(hw),
                      fname.toStdString().c_str(), "115200", SERIAL_PORT);
        ret = a.exec();
        t.join();
    } else {
        QVector<QString> fnames{"DC2C_print01.json",
                                "DC2C_print02.json",
                                "DC2C_print03.json"};
        QVector<int> templateIdx{1,2,3};
        //if (fname.startsWith("DC2C_print") && fname.endsWith(".json")) {
        //int const nrOfTemplate = fname.midRef(10, 2).toInt();
        std::thread t(updatePrinterConf, std::move(hw), templateIdx, fnames,
                     "115200", SERIAL_PORT);
        ret = a.exec();
        t.join();
        //}
    }

    return ret;
}
